@@ -17,7 +17,9 @@
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 #undef WNCK_DISABLE_DEPRECATED
@@ -50,23 +52,24 @@
 
 /* Private part of the WnckClassGroup structure */
 struct _WnckClassGroupPrivate {
-  WnckScreen *screen;
-
   char *res_class;
   char *name;
   GList *windows;
-  GHashTable *window_icon_handlers;
-  GHashTable *window_name_handlers;
 
   GdkPixbuf *icon;
   GdkPixbuf *mini_icon;
 };
 
-G_DEFINE_TYPE_WITH_PRIVATE (WnckClassGroup, wnck_class_group, G_TYPE_OBJECT);
+G_DEFINE_TYPE (WnckClassGroup, wnck_class_group, G_TYPE_OBJECT);
+#define WNCK_CLASS_GROUP_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), WNCK_TYPE_CLASS_GROUP, WnckClassGroupPrivate))
 
 /* Hash table that maps res_class strings -> WnckClassGroup instances */
 static GHashTable *class_group_hash = NULL;
 
+
+
+static void wnck_class_group_class_init  (WnckClassGroupClass *class);
+static void wnck_class_group_init        (WnckClassGroup      *class_group);
 static void wnck_class_group_finalize    (GObject             *object);
 
 enum {
@@ -92,6 +95,8 @@ wnck_class_group_class_init (WnckClassGroupClass *class)
 {
   GObjectClass *gobject_class = G_OBJECT_CLASS (class);
 
+  g_type_class_add_private (class, sizeof (WnckClassGroupPrivate));
+
   gobject_class->finalize = wnck_class_group_finalize;
 
   /**
@@ -105,7 +110,8 @@ wnck_class_group_class_init (WnckClassGroupClass *class)
                   G_OBJECT_CLASS_TYPE (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckClassGroupClass, name_changed),
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
   /**
    * WnckClassGroup::icon-changed:
@@ -118,26 +124,22 @@ wnck_class_group_class_init (WnckClassGroupClass *class)
                   G_OBJECT_CLASS_TYPE (gobject_class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (WnckClassGroupClass, icon_changed),
-                  NULL, NULL, NULL,
+                  NULL, NULL,
+                  g_cclosure_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
 }
 
 static void
 wnck_class_group_init (WnckClassGroup *class_group)
 {
-  class_group->priv = wnck_class_group_get_instance_private (class_group);
-  class_group->priv->window_icon_handlers = g_hash_table_new (g_direct_hash,
-                                                              g_direct_equal);
-  class_group->priv->window_name_handlers = g_hash_table_new (g_direct_hash,
-                                                              g_direct_equal);
-}
+  class_group->priv = WNCK_CLASS_GROUP_GET_PRIVATE (class_group);
 
-static void
-remove_signal_handler (gpointer key,
-                       gpointer value,
-                       gpointer user_data)
-{
-  g_signal_handler_disconnect (key, (gulong) value);
+  class_group->priv->res_class = NULL;
+  class_group->priv->name = NULL;
+  class_group->priv->windows = NULL;
+
+  class_group->priv->icon = NULL;
+  class_group->priv->mini_icon = NULL;
 }
 
 static void
@@ -148,52 +150,23 @@ wnck_class_group_finalize (GObject *object)
   class_group = WNCK_CLASS_GROUP (object);
 
   if (class_group->priv->res_class)
-    {
-      g_free (class_group->priv->res_class);
-      class_group->priv->res_class = NULL;
-    }
+    g_free (class_group->priv->res_class);
+  class_group->priv->res_class = NULL;
 
   if (class_group->priv->name)
-    {
-      g_free (class_group->priv->name);
-      class_group->priv->name = NULL;
-    }
+    g_free (class_group->priv->name);
+  class_group->priv->name = NULL;
 
-  if (class_group->priv->windows)
-    {
-      g_list_free (class_group->priv->windows);
-      class_group->priv->windows = NULL;
-    }
-
-  if (class_group->priv->window_icon_handlers)
-    {
-      g_hash_table_foreach (class_group->priv->window_icon_handlers,
-                            remove_signal_handler,
-                            NULL);
-      g_hash_table_destroy (class_group->priv->window_icon_handlers);
-      class_group->priv->window_icon_handlers = NULL;
-    }
-
-  if (class_group->priv->window_name_handlers)
-    {
-      g_hash_table_foreach (class_group->priv->window_name_handlers,
-                            remove_signal_handler,
-                            NULL);
-      g_hash_table_destroy (class_group->priv->window_name_handlers);
-      class_group->priv->window_name_handlers = NULL;
-    }
+  g_list_free (class_group->priv->windows);
+  class_group->priv->windows = NULL;
 
   if (class_group->priv->icon)
-    {
-      g_object_unref (class_group->priv->icon);
-      class_group->priv->icon = NULL;
-    }
+    g_object_unref (class_group->priv->icon);
+  class_group->priv->icon = NULL;
 
   if (class_group->priv->mini_icon)
-    {
-      g_object_unref (class_group->priv->mini_icon);
-      class_group->priv->mini_icon = NULL;
-    }
+    g_object_unref (class_group->priv->mini_icon);
+  class_group->priv->mini_icon = NULL;
 
   G_OBJECT_CLASS (wnck_class_group_parent_class)->finalize (object);
 }
@@ -222,7 +195,6 @@ wnck_class_group_get (const char *id)
 
 /**
  * _wnck_class_group_create:
- * @screen: a #WnckScreen.
  * @res_class: name of the resource class for the group.
  *
  * Creates a new WnckClassGroup with the specified resource class name.  If
@@ -233,8 +205,7 @@ wnck_class_group_get (const char *id)
  * matches the @res_class.
  **/
 WnckClassGroup *
-_wnck_class_group_create (WnckScreen *screen,
-                          const char *res_class)
+_wnck_class_group_create (const char *res_class)
 {
   WnckClassGroup *class_group;
 
@@ -246,7 +217,6 @@ _wnck_class_group_create (WnckScreen *screen,
 			NULL);
 
   class_group = g_object_new (WNCK_TYPE_CLASS_GROUP, NULL);
-  class_group->priv->screen = screen;
 
   class_group->priv->res_class = g_strdup (res_class ? res_class : "");
 
@@ -357,6 +327,12 @@ set_name (WnckClassGroup *class_group)
 {
   const char *new_name;
 
+  if (class_group->priv->name)
+    {
+      g_free (class_group->priv->name);
+      class_group->priv->name = NULL;
+    }
+
   new_name = get_name_from_applications (class_group);
 
   if (!new_name)
@@ -456,11 +432,11 @@ set_icon (WnckClassGroup *class_group)
   if (!icon || !mini_icon)
     {
       _wnck_get_fallback_icons (&icon,
-                                _wnck_get_default_icon_size (),
-                                _wnck_get_default_icon_size (),
+                                DEFAULT_ICON_WIDTH,
+                                DEFAULT_ICON_HEIGHT,
                                 &mini_icon,
-                                _wnck_get_default_mini_icon_size (),
-                                _wnck_get_default_mini_icon_size ());
+                                DEFAULT_MINI_ICON_WIDTH,
+                                DEFAULT_MINI_ICON_HEIGHT);
       icons_reffed = TRUE;
     }
 
@@ -484,37 +460,6 @@ set_icon (WnckClassGroup *class_group)
   g_signal_emit (G_OBJECT (class_group), signals[ICON_CHANGED], 0);
 }
 
-
-/* Handle window's icon_changed signal, update class group icon */
-static void
-update_class_group_icon (WnckWindow     *window,
-                         WnckClassGroup *class_group)
-{
-  set_icon (class_group);
-}
-
-/* Handle window's name_changed signal, update class group name */
-static void
-update_class_group_name (WnckWindow     *window,
-                         WnckClassGroup *class_group)
-{
-  set_name (class_group);
-}
-
-static void
-window_weak_notify_cb (gpointer  data,
-                       GObject  *where_the_window_was)
-{
-  WnckClassGroup *class_group;
-  WnckClassGroupPrivate *priv;
-
-  class_group = WNCK_CLASS_GROUP (data);
-  priv = class_group->priv;
-
-  g_hash_table_remove (priv->window_icon_handlers, where_the_window_was);
-  g_hash_table_remove (priv->window_name_handlers, where_the_window_was);
-}
-
 /**
  * _wnck_class_group_add_window:
  * @class_group: a #WnckClassGroup.
@@ -527,7 +472,6 @@ void
 _wnck_class_group_add_window (WnckClassGroup *class_group,
                               WnckWindow     *window)
 {
-  gulong signal_id;
 
   g_return_if_fail (WNCK_IS_CLASS_GROUP (class_group));
   g_return_if_fail (WNCK_IS_WINDOW (window));
@@ -537,24 +481,6 @@ _wnck_class_group_add_window (WnckClassGroup *class_group,
                                                window);
   _wnck_window_set_class_group (window, class_group);
 
-  signal_id = g_signal_connect (window,
-                                "icon-changed",
-                                G_CALLBACK (update_class_group_icon),
-                                class_group);
-  g_hash_table_insert (class_group->priv->window_icon_handlers,
-                       window,
-                       (gpointer) signal_id);
-
-  signal_id = g_signal_connect (window,
-                                "name-changed",
-                                G_CALLBACK (update_class_group_name),
-                                class_group);
-  g_hash_table_insert (class_group->priv->window_name_handlers,
-                       window,
-                       (gpointer) signal_id);
-
-  g_object_weak_ref (G_OBJECT (window), window_weak_notify_cb, class_group);
-
   set_name (class_group);
   set_icon (class_group);
 
@@ -575,8 +501,6 @@ void
 _wnck_class_group_remove_window (WnckClassGroup *class_group,
 				 WnckWindow     *window)
 {
-  gulong icon_handler, name_handler;
-
   g_return_if_fail (WNCK_IS_CLASS_GROUP (class_group));
   g_return_if_fail (WNCK_IS_WINDOW (window));
   g_return_if_fail (wnck_window_get_class_group (window) == class_group);
@@ -584,24 +508,6 @@ _wnck_class_group_remove_window (WnckClassGroup *class_group,
   class_group->priv->windows = g_list_remove (class_group->priv->windows,
                                               window);
   _wnck_window_set_class_group (window, NULL);
-  icon_handler = (gulong) g_hash_table_lookup (class_group->priv->window_icon_handlers,
-                                               window);
-  if (icon_handler != 0)
-    {
-      g_signal_handler_disconnect (window,
-                                   icon_handler);
-      g_hash_table_remove (class_group->priv->window_icon_handlers,
-                           window);
-    }
-  name_handler = (gulong) g_hash_table_lookup (class_group->priv->window_name_handlers,
-                                               window);
-  if (name_handler != 0)
-    {
-      g_signal_handler_disconnect (window,
-                                   name_handler);
-      g_hash_table_remove (class_group->priv->window_name_handlers,
-                           window);
-    }
 
   set_name (class_group);
   set_icon (class_group);
